var TransactWP = {};

(function() {
    var purchase_token = {}; // token used for buying single item
    var subscribe_token = {}; // subscription token
    var donate_token = {}; // donate token
    var custom_redirect = ''; // Custom redirect (after donation for instance)

    var enabledPurchase = false;
    var enabledSubscribe = false;
    var loadingPurchase = false;
    var loadingSubscribe = false;
    var purchasedInSession = false;

    var transact_modal;
    var transact_backdrop;

    TransactWP = {
        setDonateAmount: setDonateAmount,
        doPurchase: doPurchase,
        doDonate: doDonate,
        doSubscription: doSubscription
    };

    jQuery(function() {
        let nonce_value = getNonceValue();

        if (transact_params.redirect_after_donation) {
            custom_redirect = transact_params.redirect_after_donation;
        }

        getPremiumContent();

        transactApi.setFrontendUrl(transact_params.purchase_url);
        if(transact_params.theme) {
            transactApi.setThemeParams({
                bgcolor: transact_params.theme.background_color,
                tcolor: transact_params.theme.text_color
            });
        }

        if (transact_params.donation == 1) {
            getDonationTokenAjaxCall(transact_params.price || 1, nonce_value);
        } else {
            if(document.getElementById('button_purchase_single')) {
                enabledPurchase = true;
                loadingPurchase = true;

                jQuery('#button_purchase_single_loading').css('display', 'block');
                jQuery('#button_purchase_single').css('display', 'none');

                var tokenUrl = transact_params.ajaxurl + 'token/' + transact_params.post_id;
                if(transact_params.affiliate_id) {
                    tokenUrl += '/' + transact_params.affiliate_id;
                }
                jQuery.get(tokenUrl)
                    .done(function(data) {
                        loadingPurchase = false;
                        jQuery('#button_purchase_single_loading').css('display', 'none');
                        jQuery('#button_purchase_single').css('display', 'block');

                        // If purchase token loads and subscribe token is not loaded yet, display the subscribe loading spinner
                        if (enabledSubscribe && loadingSubscribe) {
                            jQuery('#button_purchase_subscription_loading').css('display', 'block');
                        }

                        console.log('got token: '+ data.token);
                        purchase_token = data.token;
                        //transactApi.setToken(data.token);
                    })
                    .fail(function(data) {
                        console.log('Failed to get Transact token');
                    });

            }

            // If subscription button is on the site, get subscription token too
            if(document.getElementById('button_purchase_subscription')) {
                enabledSubscribe = true;
                loadingSubscribe = true;

                // Only show the subscribe token spinner initially if purchase is disabled, so we don't show two spinners.
                if (!enabledPurchase) {
                    jQuery('#button_purchase_subscription_loading').css('display', 'block');
                }
                jQuery('#button_purchase_subscription').css('display', 'none');

                var subscribeTokenUrl = transact_params.ajaxurl + 'subscription/' + transact_params.post_id;
                if(transact_params.affiliate_id) {
                    subscribeTokenUrl += '/' + transact_params.affiliate_id;
                }
                jQuery.get(subscribeTokenUrl)
                    .done(function(data) {
                        loadingSubscribe = false;
                        jQuery('#button_purchase_subscription_loading').css('display', 'none');
                        jQuery('#button_purchase_subscription').css('display', 'block');

                        console.log('got subscribe_token: '+ data.token);
                        subscribe_token = data.token;
                        //transactApi.setToken(data.token);
                    })
                    .fail(function(data) {
                        console.log('Failed to get Transact token');
                    });

            }
        }

        transactApi.getCurrentPromo('en', function(arg, result) {
            var promoBlock = jQuery('#transact_promo');
            promoBlock.css('color', transact_params.theme.background_color)
            // this text comes from transact.io who we trust
            // phpcs:ignore WordPressVIPMinimum.JS.HTMLExecutingFunctions.html
            promoBlock.html(result.text);
        });
    });

    function getPremiumContent() {
        let comment_form_wrapper = jQuery('#transact-comments');

        var premiumUrl = transact_params.ajaxurl + 'premium/' + transact_params.post_id;
        jQuery.get(premiumUrl)
            .done(function(data) {
                if(data.is_premium) {
                    gtmEvent('view-premium-content', { 'viewingPastPurchase': !purchasedInSession });

                    // phpcs:ignore WordPressVIPMinimum.JS.HTMLExecutingFunctions.html
                    jQuery('#transact_content').html(jQuery.parseHTML(data.content));

                    // Get comments
                    var commentsUrl = transact_params.ajaxurl + 'comments_template/' + transact_params.post_id;
                    jQuery.get(commentsUrl)
                        .done(function(data) {
                            // phpcs:ignore WordPressVIPMinimum.JS.HTMLExecutingFunctions.append
                            comment_form_wrapper.append(jQuery.parseHTML(data.content));
                        })
                        .fail(function(data) {
                            console.log('Could not fetch comments', data);
                        });
                    
                    reloadKnownScripts();
                } else {
                    if(data.has_premium) {
                        jQuery('.transact-purchase_button').addClass('active');
                    }

                    gtmEvent('view-preview-content');
                }
            })
            .fail(function(data) {
                console.error('Could not load premium state', data);
            });
    }

    function getNonceValue() {

        let nonce_element = document.getElementById('transact_purchase_buttons_nonce');
        let nonce_value = '';
        if (nonce_element) {
            nonce_value = nonce_element.getAttribute('value');
        } else {
            console.log('getNonceValue nonce not found.');
        }
        return nonce_value;
    }

    /**
     * Gets a new donate token every time the user changes donation value
     */
    function setDonateAmount() {
        var donate = jQuery('#donate_val').val() * 100;
        if (!donate || donate < 1) {
            alert('Invalid Donation Amount');
            return;
        }
        console.log('setDonateAmount', donate);
        let nonce_value = getNonceValue();
        getDonationTokenAjaxCall(donate, nonce_value);
    }

    /**
     * AJAX call responsible of donation token
     */
    function getDonationTokenAjaxCall(donate, nonce_value) {
        jQuery('#button_purchase_donation_loading').css('display', 'block');
        jQuery('#button_purchase_donation').css('display', 'none');

        var donateUrl = transact_params.ajaxurl + 'donation/' + transact_params.post_id +
            '/' + donate;
        if(transact_params.affiliate_id) {
            donateUrl += '/' + transact_params.affiliate_id;
        }
        jQuery.get(donateUrl)
            .done(function(data) {
                jQuery('#button_purchase_donation_loading').css('display', 'none');
                jQuery('#button_purchase_donation').css('display', 'block');

                console.log('got donate_token: '+ data.token);
                donate_token = data.token;
                jQuery('#donation').removeAttr("disabled");
            })
            .fail(function(data) {
                jQuery('#donation').attr("disabled");
                console.log('Failed to get Transact donate token');
            });
    }

    /**
     * onclick for purchase button
     * Will set up purchase token on transact
     * and get the callback
     */
    function doPurchase() {
        console.log('doPurchase');
        console.log('Setting Purchase token');

        transactApi.setToken(purchase_token);
        gtmEvent('start-purchase', { 'purchaseType': 'purchase' });
        var frame = openPurchaseIFrame();
        // Call authorizeInIframe() which will load the popup,
        // passing in callback function (PurchaseFrameClosed)
        transactApi.authorizeInIframe(frame, PurchaseFrameClosed, (height) => {
            frame.style.height = height + 'px';
        });
    }

    function doDonate() {
        console.log('doDonate');
        transactApi.setToken(donate_token);
        gtmEvent('start-purchase', { 'purchaseType': 'donation' });
        var frame = openPurchaseIFrame();
        // Call authorizeInIframe() which will load the popup,
        // passing in callback function (PurchaseFrameClosed)
        transactApi.authorizeInIframe(frame, PurchaseFrameClosed, (height) => {
            frame.style.height = height + 'px';
        });
    }

    /**
     * onclick for purchase button
     * Will set up subscription token on transact
     * and get the callback
     */
    function doSubscription() {
        console.log('doSubscription');
        console.log('Setting Subscription token');
        console.log(subscribe_token);
        transactApi.setToken(subscribe_token);
        gtmEvent('start-purchase', { 'purchaseType': 'subscription' });
        var frame = openPurchaseIFrame();
        // Call authorizeInIframe() which will load the popup,
        // passing in callback function (PurchaseFrameClosed)
        transactApi.authorizeInIframe(frame, PurchaseFrameClosed, (height) => {
            frame.style.height = height + 'px';
        });
    }

    function openPurchaseIFrame() {
        transact_modal = document.createElement('div');
        transact_modal.classList.add('transact-purchase_modal');
        transact_backdrop = document.createElement('div');
        transact_backdrop.classList.add('transact-purchase_backdrop');

        var close_button = document.createElement('div');
        close_button.classList.add('transact-purchase_modal__close-button');
        jQuery([transact_backdrop, close_button]).click(function() {
            PurchaseFrameClosed(null, {});
        })

        var frame = document.createElement('iframe');
        frame.setAttribute('scrolling', 'no');

        transact_modal.appendChild(close_button);
        transact_modal.appendChild(frame);

        document.querySelector('body').appendChild(transact_modal);
        document.querySelector('body').appendChild(transact_backdrop);

        setTimeout(function() {
            transact_modal.classList.add('transact-show'); // Trigger CSS transition
            var scrollY = window.scrollY || document.documentElement.scrollTop;

            // Make sure the modal is visible
            var modalPosition = transact_modal.getBoundingClientRect();
            if(modalPosition.y < 10) {
                transact_modal.style.top = `${Math.round(scrollY) + 10}px`;
            }
        }, 50);

        return frame;
    }

    function PurchaseFrameClosed(popup, event) {
        if(transact_modal) {
            document.querySelector('body').removeChild(transact_modal);
            document.querySelector('body').removeChild(transact_backdrop);
        }

        console.log('PurchaseFrameClosed');
        console.log('PurchaseFrameClosed Event data:', event.data);

        if (event && event.data) {
            console.log(event.data);
            var validation_data = {};
            validation_data.post_id = transact_params.post_id;
            validation_data.token = event.data.t;

            jQuery.post(transact_params.ajaxurl + 'verify', validation_data)
                .done(function(resp_data) {
                    gtmEvent('complete-purchase');

                    purchasedInSession = true;
                    console.log('Success Response data:', resp_data);
                    // Handles cookie
                    handleCookies(validation_data, resp_data);
                    // if custom redirect, send user to it, otherwise reload
                    if (custom_redirect.length > 0) {
                        // we set redirect here
                        // phpcs:ignore WordPressVIPMinimum.JS.Window.location
                        window.location = custom_redirect;
                    } else {
                        getPremiumContent();
                        // location.reload();
                    }
                })
                .fail(function(resp_data) {
                    console.log('Error Response data:', resp_data);
                    jQuery('#button_purchase').html('purchase failed');
                })
                .always(function() {
                    console.log( "finished" );
                });
        } else {
            gtmEvent('cancel-purchase');
        }
    }

    function handleCookies(validation_data, resp_data)
    {
        console.log(resp_data, resp_data.decoded);
        // Set or Update Cookie
        if (resp_data.subscription == '1' || resp_data.subscription == 1) {
            handleSubscriptionCookies(resp_data);
        } else {
            handlePurchaseCookies(validation_data, resp_data);
        }
    }

    function handleSubscriptionCookies(resp_data)
    {
        console.log('setting wp_subscription_transact_ cookie', resp_data);
        var cookie = getCookie('wp_subscription_transact_');
        if (cookie != '') {
            cookie_array = JSON.parse(cookie);
            var new_cookie = {};
            new_cookie['expiration']  = resp_data.decoded.sub_expires;
            new_cookie['uid'] = resp_data.decoded.uid;
            new_cookie['bid'] = resp_data.decoded.bid;
            cookie_array.push(new_cookie);
            setCookie('wp_subscription_transact_', JSON.stringify(cookie_array), 365);
        } else {
            var new_cookie = {};
            new_cookie['expiration']  = resp_data.decoded.sub_expires;
            new_cookie['uid'] = resp_data.decoded.uid;
            new_cookie['bid'] = resp_data.decoded.bid;
            var cookies = [];
            cookies.push(new_cookie);
            setCookie('wp_subscription_transact_', JSON.stringify(cookies), 365);
        }
    }

    function handlePurchaseCookies(validation_data, resp_data)
    {
        console.log('setting wp_transact_ cookie');
        var cookie = getCookie('wp_transact_');
        if (cookie != '') {
            cookie_array = JSON.parse(cookie);
            var new_cookie = {};
            new_cookie['id']  = validation_data.post_id;
            new_cookie['uid'] = resp_data.decoded.uid;
            cookie_array.push(new_cookie);
            setCookie('wp_transact_', JSON.stringify(cookie_array), 365);

        } else {
            var new_cookie = {};
            new_cookie['id']  = validation_data.post_id;
            new_cookie['uid'] = resp_data.decoded.uid;
            var cookies = [];
            cookies.push(new_cookie);

            setCookie('wp_transact_', JSON.stringify(cookies), 365);
        }
    }

    function getCookie(cname) {
        var name = cname + "=";
        var ca = document.cookie.split(';');
        for(var i = 0; i < ca.length; i++) {
            var c = ca[i];
            while (c.charAt(0) == ' ') {
                c = c.substring(1);
            }
            if (c.indexOf(name) == 0) {
                return c.substring(name.length, c.length);
            }
        }
        return "";
    }

    function setCookie(cname, cvalue, exdays) {
        var d = new Date();
        d.setTime(d.getTime() + (exdays*24*60*60*1000));
        var expires = "expires="+d.toUTCString();
        document.cookie = cname + "=" + cvalue + ";" + expires + ";path=/";
    }

    function reloadKnownScripts() {
        document.body.querySelectorAll('script').forEach(script => {
            if(script.src.match(/tiled-gallery/)) {
                var newScript = document.createElement('script');
                newScript.src = script.src;
                script.parentNode.removeChild(script);
                // phpcs:ignore WordPressVIPMinimum.JS.HTMLExecutingFunctions.append
                document.body.append(newScript);
            }
        });
    }
    
    function gtmEvent(name, addtlParams) {
        if(window.dataLayer !== undefined) {
            window.dataLayer.push(
                Object.assign(
                    {
                        'event': name,
                        'location': document.URL,
                        'postId': transact_params.post_id,
                        'postPrice': transact_params.price,
                    },
                    addtlParams || {}
                )
            );
        }
    }
})();